;Filename: TCfg.s
;Part: ADuC703x.
;Created: 12/04/2006
;Author: Eckart Hartmann of Analog Devices
;This file is in beta sampling.

;This software is supplied subject our standard software license agreement.
;A non-binding summary is as follows:
;"You may use the software in your own code at your own risk.
;" ADI inc disclaims any warranty or liability of its use".
;The full standard software license agreement is available on request and
; remains the binding agreement.

#include        "..\Fla\ADuC703xi.equ"
Z_TnLD      DEFINE  0x300
Z_TnVAL     DEFINE  0x304
Z_TnCON     DEFINE  0x308
Z_TnCLRI    DEFINE  0x30c
Z_TnCAP     DEFINE  0x310

; TCfg==========Configures timers.
; C Function prototype: int TCfg(int iT, int iScale, int iClk);
;iT:{T_T0,T_T1,T_T2,T_T3,T_T4}
;iScale:{1,16,256,32768}
;iClk:{T_HCLK,T_LV32K,T_XTAL,T_HV32K}
; Description of Function: Configures clocking of timers T0 to T4.
; User interface:
;   Set iT to desired Timer, 0, 1, 2 or 4.
;   Set iScale to a prescale value of 1, 16, 256 or 32768 as appropriate.
;   Set iClk to the desired clock source.   (Not T3)
;       0 = T_HCLK         2 = T_XTAL   (Not for T4. GPIO8 for T1.)
;       1 = T_LV32K        3 = T_HV32K  (Not for T4. GPIO5 for T1.)
;   Call TCfg(iT, iScale, iClk).
;       Timer iT is stopped in periodic mode and prescale set to iScale
;           and iClk clock source selected.
;   Returns previous TnCON value in r0.
; Robustness:
;  This function looses the settings done by TMod().
;  FreeRunning mode not supported. Use periodic mode with reload value
;   of full scale instead.
;  Certain features are not available for certain timers. Refer to the
;  data sheet for more information.
; Side effects: Corrupts r0 to r3.

                RSEG CODE
                MODULE TCfg
                PUBLIC TCfg
                CODE32

TCfg:   stmfd   sp!,{r4}
        subs    r1,r1,#16               ;Compare r1 to 16
        movlt   r4,#0x40                ; if lt set /1 and periodic
        moveq   r4,#0x44                ; if 16 set /16 and periodic
        movgt   r4,#0x48                ; if gt set /256 and periodic.
        cmpgt   r1,#256                 ;Compare r1 to 256
        movgt   r4,#0x4f                ; if gt set /32768 and periodic.
        cmp     r0,#3                   ;if(iT==3)
        biceq   r4,r4,#3                ; clear bits 0-1.

        and     r2,r2,#3
        orr     r4,r4,r2,lsl#9          ;Add iClk into r4

        mov     r3,#0xff0000            ;r3 = Z_BASE;
        orr     r3,r3,#0xff000000
        cmp     r0,#4                   ;Ensure valid timer.
        andne   r0,r0,#3
        orr     r3,r3,r0,lsl#5          ;Z_BASE |= r0<<5;
        cmp     r0,#0                   ;If T0
        addeq   r3,r3,#4                ; Z_BASE += 4.

        ldr     r0,[r3,#Z_TnCON]        ;Get TnCON.
        str     r4,[r3,#Z_TnCON]        ;Write to TnCON
        ldmfd   sp!,{r4}
TCfgR:  bx      lr

                 ENDMOD  TCfg
                 END

